using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace PrivateDataCommon
{
    /// <summary>
    /// Encrypted content - represents something encrypted; besides encrypted
    /// data, it also contains the initialization vector used for encryption.
    /// </summary>
    public class EncryptedContent : Content
    {
        /// <summary>
        /// Constructs an EncryptedContent
        /// </summary>
        /// <param name="encryptedData">
        /// Stream containing the encrypted data. This stream is just stored and
        /// made available to the class' callers.
        /// </param>
        /// <param name="iv">Initialization vector used for encryption</param>
        public EncryptedContent(Stream encryptedData, IV iv)
            : base(encryptedData)
        {
            this.iv = iv;
        }

        /// <summary>
        /// Initialization vector
        /// </summary>
        public IV IV
        {
            get
            {
                return iv;
            }
        }

        private IV iv;
    }
}
