using System;
using System.Collections.Generic;
using System.Text;

namespace PrivateDataCommon
{
    public class SlowOperationController
    {
        public bool CancelRequested
        {
            get
            {
                lock (this)
                {
                    return cancel;
                }
            }

            set
            {
                lock (this)
                {
                    cancel = value;
                }
            }
        }

        public void ThrowIfCancelRequested()
        {
            if (CancelRequested)
            {
                throw new Exceptions.OperationCancelledException();
            }
        }

        public void StartProgress(ulong progressMax)
        {
            OnProgressStarted(new ProgressStartedArgs(progressMax));
        }

        public void ReportProgress(ulong progress)
        {
            OnProgressReported(new ProgressReportedArgs(progress));
        }

        public void FinishProgress()
        {
            OnProgressFinished(EventArgs.Empty);
        }

        protected void OnProgressStarted(ProgressStartedArgs e)
        {
            if (ProgressStarted != null)
            {
                ProgressStarted(this, e);
            }
        }

        protected void OnProgressReported(ProgressReportedArgs e)
        {
            if (ProgressReported != null)
            {
                ProgressReported(this, e);
            }
        }

        protected void OnProgressFinished(EventArgs e)
        {
            if (ProgressFinished != null)
            {
                ProgressFinished(this, e);
            }
        }

        private bool cancel;

        public event EventHandler<ProgressStartedArgs> ProgressStarted;
        public event EventHandler<ProgressReportedArgs> ProgressReported;
        public event EventHandler ProgressFinished;
    }
}
