using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using PrivateDataCommon;

namespace PrivateDataCore
{
    /// <summary>
    /// Interface for the functionality that items coming from files must have.
    /// </summary>
    public interface IFileItem
    {
        /// <summary>
        /// Imports the contents of a file into the item
        /// </summary>
        /// <param name="stream">
        /// Stream to read the item's contents from.
        /// The string should be readable.
        /// The item takes ownership of the stream (i.e. it can hold it open
        /// for a while and close it as it sees fit).
        /// </param>
        void Import(Stream file);

        /// <summary>
        /// Exports the contents of the item to a file
        /// </summary>
        /// <param name="stream">
        /// Stream to write the item's contents to.
        /// The string should be writable.
        /// </param>
        /// <param name="controller">Slow operation controller</param>
        void Export(Stream file, SlowOperationController controller);

        /// <summary>
        /// Returns true if the item's contents have been loaded through Import.
        /// This can be used to tell whether the item needs to be saved into
        /// the storage or not (it needs to be saved if it's been imported;
        /// otherwise it must be coming from the storage).
        /// </summary>
        bool IsImported
        {
            get;
        }
    }
}
