using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using PrivateDataCommon;

namespace PrivateDataCore.ItemTypes
{
    public class BankAccount : Item
    {
        private BankAccount(Metadata metadata)
            : base(metadata)
        {
            // Nothing to do here
        }

        private BankAccount(ItemID id, Metadata metadata, Content content)
            : base(id, metadata)
        {
            Stream stream = content.DetachData();
            try
            {
                financialInstitution = Utils.ReadStringFromStream(stream);
                accountType = Utils.ReadStringFromStream(stream);
                accountNumber = Utils.ReadStringFromStream(stream);
                pin = Utils.ReadStringFromStream(stream);
                customerSupport = Utils.ReadStringFromStream(stream);
                routingNumber = Utils.ReadStringFromStream(stream);
                url = Utils.ReadStringFromStream(stream);
                username = Utils.ReadStringFromStream(stream);
                password = Utils.ReadStringFromStream(stream);
                notes = Utils.ReadStringFromStream(stream);
            }
            finally
            {
                stream.Close();
            }
        }

        public override Content GenerateContent()
        {
            MemoryStream memoryStream = new MemoryStream();

            Utils.WriteStringToStream(financialInstitution, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(accountType, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(accountNumber, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(pin, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(customerSupport, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(routingNumber, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(url, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(username, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(password, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(notes, memoryStream, false/*estimate*/);

            return new Content(memoryStream);
        }

        /// <summary>
        /// Financial institution
        /// </summary>
        public string FinancialInstitution
        {
            get
            {
                return financialInstitution;
            }

            set
            {
                financialInstitution = value;
            }
        }

        /// <summary>
        /// Type of the account
        /// </summary>
        public string AccountType
        {
            get
            {
                return accountType;
            }

            set
            {
                accountType = value;
            }
        }

        /// <summary>
        /// Account number
        /// </summary>
        public string AccountNumber
        {
            get
            {
                return accountNumber;
            }

            set
            {
                accountNumber = value;
            }
        }

        /// <summary>
        /// PIN code
        /// </summary>
        public string PIN
        {
            get
            {
                return pin;
            }

            set
            {
                pin = value;
            }
        }

        /// <summary>
        /// Customer support phone number
        /// </summary>
        public string CustomerSupport
        {
            get
            {
                return customerSupport;
            }

            set
            {
                customerSupport = value;
            }
        }

        /// <summary>
        /// Routing number
        /// </summary>
        public string RoutingNumber
        {
            get
            {
                return routingNumber;
            }

            set
            {
                routingNumber = value;
            }
        }

        /// <summary>
        /// URL
        /// </summary>
        public string URL
        {
            get
            {
                return url;
            }

            set
            {
                url = value;
            }
        }

        /// <summary>
        /// Username
        /// </summary>
        public string Username
        {
            get
            {
                return username;
            }

            set
            {
                username = value;
            }
        }

        /// <summary>
        /// Password
        /// </summary>
        public string Password
        {
            get
            {
                return password;
            }

            set
            {
                password = value;
            }
        }

        private string financialInstitution = String.Empty;
        private string accountType = String.Empty;
        private string accountNumber = String.Empty;
        private string pin = String.Empty;
        private string customerSupport = String.Empty;
        private string routingNumber = String.Empty;
        private string url = String.Empty;
        private string username = String.Empty;
        private string password = String.Empty;
    }
}
