using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using PrivateDataCommon;

namespace PrivateDataCore.ItemTypes
{
    public class CellPhone : Item
    {
        private CellPhone(Metadata metadata)
            : base(metadata)
        {
            // Nothing to do here
        }

        private CellPhone(ItemID id, Metadata metadata, Content content)
            : base(id, metadata)
        {
            Stream stream = content.DetachData();
            try
            {
                manufacturer = Utils.ReadStringFromStream(stream);
                model = Utils.ReadStringFromStream(stream);
                phoneNumber = Utils.ReadStringFromStream(stream);
                pin = Utils.ReadStringFromStream(stream);
                securityCode = Utils.ReadStringFromStream(stream);
                imei = Utils.ReadStringFromStream(stream);
                customerSupport = Utils.ReadStringFromStream(stream);
                url = Utils.ReadStringFromStream(stream);
                username = Utils.ReadStringFromStream(stream);
                password = Utils.ReadStringFromStream(stream);
                notes = Utils.ReadStringFromStream(stream);
            }
            finally
            {
                stream.Close();
            }
        }

        public override Content GenerateContent()
        {
            MemoryStream memoryStream = new MemoryStream();

            Utils.WriteStringToStream(manufacturer, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(model, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(phoneNumber, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(pin, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(securityCode, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(imei, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(customerSupport, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(url, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(username, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(password, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(notes, memoryStream, false/*estimate*/);

            return new Content(memoryStream);
        }

        /// <summary>
        /// Manufacturer of the phone
        /// </summary>
        public string Manufacturer
        {
            get
            {
                return manufacturer;
            }

            set
            {
                manufacturer = value;
            }
        }

        /// <summary>
        /// Model of the phone
        /// </summary>
        public string Model
        {
            get
            {
                return model;
            }

            set
            {
                model = value;
            }
        }

        /// <summary>
        /// Phone number
        /// </summary>
        public string PhoneNumber
        {
            get
            {
                return phoneNumber;
            }

            set
            {
                phoneNumber = value;
            }
        }

        /// <summary>
        /// PIN code
        /// </summary>
        public string PIN
        {
            get
            {
                return pin;
            }

            set
            {
                pin = value;
            }
        }

        /// <summary>
        /// Security code of the phone
        /// </summary>
        public string SecurityCode
        {
            get
            {
                return securityCode;
            }

            set
            {
                securityCode = value;
            }
        }

        /// <summary>
        /// IMEI number
        /// </summary>
        public string Imei
        {
            get
            {
                return imei;
            }

            set
            {
                imei = value;
            }
        }

        /// <summary>
        /// Customer support phone number
        /// </summary>
        public string CustomerSupport
        {
            get
            {
                return customerSupport;
            }
            set
            {
                customerSupport = value;
            }
        }

        /// <summary>
        /// URL
        /// </summary>
        public string URL
        {
            get
            {
                return url;
            }

            set
            {
                url = value;
            }
        }

        /// <summary>
        /// Username
        /// </summary>
        public string Username
        {
            get
            {
                return username;
            }

            set
            {
                username = value;
            }
        }

        /// <summary>
        /// Password
        /// </summary>
        public string Password
        {
            get
            {
                return password;
            }

            set
            {
                password = value;
            }
        }

        private string manufacturer = String.Empty;
        private string model = String.Empty;
        private string phoneNumber = String.Empty;
        private string pin = String.Empty;
        private string securityCode = String.Empty;
        private string imei = String.Empty;
        private string customerSupport = String.Empty;
        private string url = String.Empty;
        private string username = String.Empty;
        private string password = String.Empty;
    }
}
