using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using PrivateDataCommon;

namespace PrivateDataCore.ItemTypes
{
    public class CreditCard : Item
    {
        private CreditCard(Metadata metadata)
            : base(metadata)
        {
            // Nothing to do here
        }

        private CreditCard(ItemID id, Metadata metadata, Content content)
            : base(id, metadata)
        {
            Stream stream = content.DetachData();
            try
            {
                financialInstitution = Utils.ReadStringFromStream(stream);
                cardType = Utils.ReadStringFromStream(stream);
                cardNumber = Utils.ReadStringFromStream(stream);
                nameOnCard = Utils.ReadStringFromStream(stream);
                expiration.Month = Utils.ReadIntFromStream(stream);
                expiration.Year = Utils.ReadIntFromStream(stream);
                cardVerificationCode = Utils.ReadStringFromStream(stream);
                pin = Utils.ReadStringFromStream(stream);
                customerSupport = Utils.ReadStringFromStream(stream);
                url = Utils.ReadStringFromStream(stream);
                username = Utils.ReadStringFromStream(stream);
                password = Utils.ReadStringFromStream(stream);
                notes = Utils.ReadStringFromStream(stream);
            }
            finally
            {
                stream.Close();
            }
        }

        public override Content GenerateContent()
        {
            MemoryStream memoryStream = new MemoryStream();

            Utils.WriteStringToStream(financialInstitution, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(cardType, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(cardNumber, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(nameOnCard, memoryStream, false/*estimate*/);
            Utils.WriteIntToStream(expiration.Month, memoryStream, false/*estimate*/);
            Utils.WriteIntToStream(expiration.Year, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(cardVerificationCode, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(pin, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(customerSupport, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(url, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(username, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(password, memoryStream, false/*estimate*/);
            Utils.WriteStringToStream(notes, memoryStream, false/*estimate*/);

            return new Content(memoryStream);
        }

        /// <summary>
        /// Financial institution
        /// </summary>
        public string FinancialInstitution
        {
            get
            {
                return financialInstitution;
            }
            set
            {
                financialInstitution = value;
            }
        }

        /// <summary>
        /// Type of the account
        /// </summary>
        public string CardType
        {
            get
            {
                return cardType;
            }
            set
            {
                cardType = value;
            }
        }

        /// <summary>
        /// Account number
        /// </summary>
        public string CardNumber
        {
            get
            {
                return cardNumber;
            }
            set
            {
                cardNumber = value;
            }
        }

        /// <summary>
        /// Name on the card
        /// </summary>
        public string NameOnCard
        {
            get
            {
                return nameOnCard;
            }
            set
            {
                nameOnCard = value;
            }
        }

        /// <summary>
        /// Expiration date of the card
        /// </summary>
        public ExpirationDate Expiration
        {
            get
            {
                return expiration;
            }

            set
            {
                expiration = value;
            }
        }

        /// <summary>
        /// Card verification code
        /// </summary>
        public string CardVerificationCode
        {
            get
            {
                return cardVerificationCode;
            }
            set
            {
                cardVerificationCode = value;
            }
        }

        /// <summary>
        /// PIN code
        /// </summary>
        public string PIN
        {
            get
            {
                return pin;
            }
            set
            {
                pin = value;
            }
        }

        /// <summary>
        /// Customer support phone number
        /// </summary>
        public string CustomerSupport
        {
            get
            {
                return customerSupport;
            }
            set
            {
                customerSupport = value;
            }
        }

        /// <summary>
        /// URL
        /// </summary>
        public string URL
        {
            get
            {
                return url;
            }

            set
            {
                url = value;
            }
        }

        /// <summary>
        /// Username
        /// </summary>
        public string Username
        {
            get
            {
                return username;
            }

            set
            {
                username = value;
            }
        }

        /// <summary>
        /// Password
        /// </summary>
        public string Password
        {
            get
            {
                return password;
            }

            set
            {
                password = value;
            }
        }

        public struct ExpirationDate
        {
            public ExpirationDate(DateTime date)
            {
                month = date.Month;
                year = date.Year;
            }

            public int Month
            {
                get
                {
                    return month;
                }

                set
                {
                    month = value;
                }
            }

            public int Year
            {
                get
                {
                    return year;
                }

                set
                {
                    year = value;
                }
            }

            private int month;
            private int year;
        }

        private string financialInstitution = String.Empty;
        private string cardType = String.Empty;
        private string cardNumber = String.Empty;
        private string nameOnCard = String.Empty;
        private ExpirationDate expiration = new ExpirationDate(DateTime.Now);
        private string cardVerificationCode = String.Empty;
        private string pin = String.Empty;
        private string customerSupport = String.Empty;
        private string url = String.Empty;
        private string username = String.Empty;
        private string password = String.Empty;
    }
}
