using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using PrivateDataCommon;

namespace PrivateDataCore.ItemTypes
{
    public class FileItem : Item, IFileItem // TODO: IDisposable
    {
        private FileItem(Metadata metadata)
            : base(metadata)
        {
            // Nothing to do here
        }

        private FileItem(ItemID id, Metadata metadata, Content content)
            : base(id, metadata)
        {
            stream = content.DetachData();
        }

        public override Content GenerateContent()
        {
            Stream contentStream = stream;
            stream = null;

            return new Content(contentStream);
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (stream != null)
                {
                    // HACK: The CryptoStream crashes if it's closed without
                    //       ever having read from it, so we need to read something
                    //       from it in order to be able to close it properly.
                    Utils.CopyWholeStreamToStream(stream, Stream.Null, new SlowOperationController());

                    stream.Close();
                    stream = null;
                }
            }
            base.Dispose(disposing);
        }

        #region IFileItem Members

        public void Import(Stream file)
        {
            if (stream != null)
            {
                // Why is Import being called on an already loaded FileItem?
                throw new Exception(); // TODO: Fix exceptions
            }

            stream = file;
            isImported = true;
        }

        public void Export(Stream file, SlowOperationController controller)
        {
            if (stream == null)
            {
                // Why is Export being called on a not-yet-loaded (or already
                // used) FileItem?
                throw new Exception(); // TODO: Fix exceptions
            }

            // If we can seek our stream, let's start from the beginning
            if (stream.CanSeek)
            {
                stream.Seek(0, SeekOrigin.Begin);
            }

            Utils.CopyWholeStreamToStream(stream, file, controller);

            // If we have a one-use stream, it's of no longer use
            if (!stream.CanSeek)
            {
                stream.Close();
                stream = null;
            }
        }

        public bool IsImported
        {
            get
            {
                return isImported;
            }
        }

        #endregion

        private Stream stream;
        private bool isImported;
    }
}
