using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using PrivateDataCommon;

namespace PrivateDataCore.ItemTypes
{
    public class Picture : Item, IFileItem // TODO: IDisposable
    {
        private Picture(Metadata metadata)
            : base(metadata)
        {
            // Nothing to do here
        }

        private Picture(ItemID id, Metadata metadata, Content content)
            : base(id, metadata)
        {
            Stream stream = content.DetachData();
            try
            {
                if (stream.CanSeek)
                {
                    pictureStream = stream;
                    stream = null;
                }
                else
                {
                    pictureStream = new MemoryStream();
                    Utils.CopyWholeStreamToStream(
                        stream,
                        pictureStream,
                        new SlowOperationController()); // TODO: Can we use a real one?
                }
            }
            finally
            {
                if (stream != null)
                {
                    stream.Close();
                }
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                pictureStream.Close();
            }
            base.Dispose(disposing);
        }

        /// <summary>
        /// Returns the stream containing the picture data
        /// </summary>
        public Stream PictureStream
        {
            get
            {
                return pictureStream;
            }
        }

        public override Content GenerateContent()
        {
            return new Content(PictureStream);
        }

        #region IFileItem Members

        public void Import(Stream file)
        {
            if (pictureStream != null)
            {
                // Why is Import being called on an already loaded Picture?
                throw new Exception(); // TODO: Fix exceptions
            }

            pictureStream = file;
            isImported = true;
        }

        public void Export(Stream file, SlowOperationController controller)
        {
            if (pictureStream == null)
            {
                // Why is Export being called on a not-yet-loaded (or already
                // used) Picture?
                throw new Exception(); // TODO: Fix exceptions
            }

            // If we can seek our stream, let's start from the beginning
            if (pictureStream.CanSeek)
            {
                pictureStream.Seek(0, SeekOrigin.Begin);
            }

            Utils.CopyWholeStreamToStream(pictureStream, file, controller);

            // If we have a one-use stream, it's of no longer use
            if (!pictureStream.CanSeek)
            {
                pictureStream.Close();
                pictureStream = null;
            }
        }

        public bool IsImported
        {
            get
            {
                return isImported;
            }
        }

        #endregion

        public bool HasStream
        {
            get
            {
                return pictureStream != null;
            }
        }

        private Stream pictureStream;
        private bool isImported;
    }
}

