using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using PrivateDataCommon;

namespace PrivateDataCore
{
    /// <summary>
    /// Metadata describing a particular private item
    /// </summary>
    public class Metadata
    {
        /// <summary>
        /// Constructs Metadata from an item's name and type
        /// </summary>
        /// <param name="name">Name describing the item</param>
        /// <param name="type">Type of the item</param>
        public Metadata(string name, ItemType type)
        {
            this.name = name;
            this.type = type;
        }

        /// <summary>
        /// Constructs Metadata from saved content
        /// </summary>
        /// <param name="metadataContent">Content containing the metadata</param>
        public Metadata(Content metadataContent)
        {
            Stream stream = metadataContent.DetachData();

            name = Utils.ReadStringFromStream(stream);
            
            string typeString = Utils.ReadStringFromStream(stream);
            type = (ItemType)Enum.Parse(
                typeof(ItemType),
                typeString,
                true/*ignoreCase*/);
            
            fileName = Utils.ReadStringFromStream(stream);
        }

        /// <summary>
        /// Converts the metadata to content, ready for encryption and storage
        /// </summary>
        /// <returns>Content containing the metadata</returns>
        public Content ToContent()
        {
            MemoryStream metadataStream = new MemoryStream();

            Utils.WriteStringToStream(name, metadataStream, false/*estimate*/);
            Utils.WriteStringToStream(type.ToString(), metadataStream, false/*estimate*/);
            Utils.WriteStringToStream(fileName, metadataStream, false/*estimate*/);

            return new Content(metadataStream);
        }

        /// <summary>
        /// Name describing the item 
        /// </summary>
        public string Name
        {
            get
            {
                return name;
            }

            set
            {
                name = value;
            }
        }

        /// <summary>
        /// Item type
        /// </summary>
        public ItemType Type
        {
            get
            {
                return type;
            }
        }

        /// <summary>
        /// For items coming from files, the name of the original file.
        /// For all other items - String.Empty.
        /// </summary>
        public string FileName
        {
            get
            {
                return fileName;
            }

            set
            {
                fileName = value;
            }
        }

        private string name;
        private ItemType type;
        private string fileName = String.Empty;
    }
}
