using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using PrivateDataCommon;

namespace PrivateData
{
    public class AsyncOperationUI
    {
        public AsyncOperationUI(
            AsyncOperation operation,
            Control control,
            EventHandler successHandler,
            EventHandler cancellationHandler,
            EventHandler failureHandler)
        {
            this.operation = operation;
            this.control = control;
            this.successHandler = successHandler;
            this.cancellationHandler = cancellationHandler;
            this.failureHandler = failureHandler;

            operation.Succeeded += new EventHandler(operation_Succeeded);
            operation.Cancelled += new EventHandler(operation_Cancelled);
            operation.Failed += new EventHandler(operation_Failed);

            progress = new ProgressDialog(operation);
            progress.Show();
        }

        private void operation_Succeeded(object sender, EventArgs e)
        {
            control.BeginInvoke(new EventHandler(Succeeded), sender, e);
        }

        private void operation_Cancelled(object sender, EventArgs e)
        {
            control.BeginInvoke(new EventHandler(Cancelled), sender, e);
        }

        private void operation_Failed(object sender, EventArgs e)
        {
            control.BeginInvoke(new EventHandler(Failed), sender, e);
        }

        private void Succeeded(object sender, EventArgs e)
        {
            progress.Close();
            successHandler(sender, e);
        }

        private void Cancelled(object sender, EventArgs e)
        {
            progress.Close();
            cancellationHandler(sender, e);
        }

        private void Failed(object sender, EventArgs e)
        {
            progress.Close();
            failureHandler(sender, e);
        }

        private AsyncOperation operation;
        private Control control;
        private EventHandler successHandler;
        private EventHandler cancellationHandler;
        private EventHandler failureHandler;

        private ProgressDialog progress;
    }
}
