using System;
using System.Collections.Generic;
using System.Text;

using PrivateDataCommon;
using PrivateDataCore;
using PrivateDataCore.Exceptions;

namespace PrivateData.AsyncOperations
{
    internal class ChangePasswordOperation : AsyncOperation
    {
        public ChangePasswordOperation(
            DataManager dataManager,
            string oldPassword,
            string newPassword,
            uint newIterationCount)
        {
            this.dataManager = dataManager;
            this.oldPassword = oldPassword;
            this.newPassword = newPassword;
            this.newIterationCount = newIterationCount;
        }

        public override string Message
        {
            get
            {
                return Properties.Resources.ChangePasswordOperationMessage;
            }
        }

        public override bool Cancellable
        {
            get
            {
                // Changing the password should not be cancellable; otherwise
                // we risk having the new password information stored into the
                // file but not giving the data manager a chance to extract
                // the key from the encryption header
                return false;
            }
        }

        protected override void Execute(SlowOperationController controller)
        {
            try
            {
                dataManager.ChangePassword(
                    oldPassword,
                    newPassword,
                    newIterationCount,
                    controller);
            }
            catch(WrongPasswordException)
            {
                incorrectPassword = true;
                throw;
            }
        }

        public bool IncorrectPassword
        {
            get
            {
                return incorrectPassword;
            }
        }

        private DataManager dataManager;
        private string oldPassword;
        private string newPassword;
        private uint newIterationCount;

        private bool incorrectPassword;
    }
}
