using System;
using System.Collections.Generic;
using System.Text;

using PrivateDataCommon;
using PrivateDataCore;

namespace PrivateData.AsyncOperations
{
    internal class CreateStorageOperation : AsyncOperation
    {
        public CreateStorageOperation(
            DataManager dataManager,
            string dataPath,
            string password,
            uint iterationCount)
        {
            this.dataManager = dataManager;
            this.dataPath = dataPath;
            this.password = password;
            this.iterationCount = iterationCount;
        }

        public override string Message
        {
            get
            {
                return Properties.Resources.CreateStorageOperationMessage;
            }
        }

        public override bool Cancellable
        {
            get
            {
                // Create storage need not be cancellable. Let's not give the
                // user a chance to create a partially created (thus corrupted)
                // data file, creating a file but not giving the data manager
                // a chance to extract the key from the encryption header, etc.
                return false;
            }
        }

        protected override void Execute(SlowOperationController controller)
        {
            dataManager.CreateStorage(
                dataPath,
                password,
                iterationCount,
                controller);
        }

        private DataManager dataManager;
        private string dataPath;
        private string password;
        private uint iterationCount;
    }
}
