using System;
using System.Collections.Generic;
using System.Text;

using PrivateDataCommon;
using PrivateDataCore;

namespace PrivateData.AsyncOperations
{
    internal class DeleteItemOperation : AsyncOperation
    {
        public DeleteItemOperation(DataManager dataManager, ItemID item)
        {
            this.dataManager = dataManager;
            
            items = new Stack<ItemID>();
            StackItem(item);
        }

        private void StackItem(ItemID item)
        {
            items.Push(item);

            if (dataManager.GetMetadata(item).Type == ItemType.Folder)
            {
                ItemID[] children = dataManager.GetFolderItems(item);
                foreach (ItemID child in children)
                {
                    StackItem(child);
                }
            }
        }

        public override string Message
        {
            get
            {
                return Properties.Resources.DeleteItemOperationMessage;
            }
        }

        public override bool Cancellable
        {
            get
            {
                // Deleting a folder full of items ought to be cancellable
                return true;
            }
        }

        protected override void Execute(SlowOperationController controller)
        {
            controller.StartProgress((uint)items.Count);
            uint progress = 0;
            while (items.Count > 0)
            {
                dataManager.DeleteItem(items.Pop(), controller);
                controller.ReportProgress(++progress);
            }
            controller.FinishProgress();
        }

        private DataManager dataManager;
        private Stack<ItemID> items;
    }
}
