using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

using PrivateDataCommon;
using PrivateDataCore;

namespace PrivateData
{
    public partial class CreateItemDialog : Form
    {
        public CreateItemDialog(DataManager dataManager, ItemID folder, bool createFolder)
        {
            this.dataManager = dataManager;
            this.folder = folder;
            this.createFolder = createFolder;

            InitializeComponent();

            SuspendLayout();
            if (createFolder)
            {
                okButton.Top = typeLabel.Top;
                cancelButton.Top = typeLabel.Top;
                typeComboBox.Visible = false;
                typeLabel.Visible = false;
            }
            else
            {
                types = new Dictionary<string, ItemType>();
                types.Add("Bank account", ItemType.BankAccount);
                types.Add("Credit card", ItemType.CreditCard);
                types.Add("Password", ItemType.GenericPassword);
                types.Add("Web account", ItemType.WebAccount);
                types.Add("E-mail", ItemType.Email);
                types.Add("Note", ItemType.Note);
                types.Add("Cell phone", ItemType.CellPhone);
                types.Add("Picture", ItemType.Picture);
                types.Add("File", ItemType.File);

                foreach (string text in types.Keys)
                {
                    typeComboBox.Items.Add(text);
                }
            }
            ResumeLayout();

            nameTextBox.Focus();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            ItemType type;
            if (createFolder)
            {
                type = ItemType.Folder;
                Metadata metadata = new Metadata(nameTextBox.Text, type);
                Item item = Item.Create(metadata);
                AsyncOperations.StoreItemOperation storeItem =
                    new AsyncOperations.StoreItemOperation(
                        dataManager,
                        item,
                        folder);

                new AsyncOperationUI(
                    storeItem,
                    this,
                    new EventHandler(storeItem_Succeeded),
                    new EventHandler(storeItem_Cancelled),
                    new EventHandler(storeItem_Failed));

                storeItem.BeginExecute();
            }
            else
            {
                type = types[typeComboBox.Text];
                Metadata metadata = new Metadata(nameTextBox.Text, type);
                Item item = Item.Create(metadata);

                if (item is IFileItem)
                {
                    OpenFileDialog openFileDialog = new OpenFileDialog();
                    switch (item.Type)
                    {
                        case ItemType.Picture:
                                openFileDialog.Filter = Properties.Resources.PictureFileFilter;
                                break;
                        case ItemType.File:
                                openFileDialog.Filter = Properties.Resources.AllFilesFileFilter;
                                break;
                        default:
                            throw new Exception(); // TODO: Fix exceptions

                    }
                    
                    if (openFileDialog.ShowDialog() == DialogResult.OK)
                    {
                        ((IFileItem)item).Import(
                            File.Open(
                                openFileDialog.FileName,
                                FileMode.Open,
                                FileAccess.Read,
                                FileShare.Read));
                        item.FileName = openFileDialog.FileName;
                    }
                    else
                    {
                        // User cancelled
                        DialogResult = DialogResult.Cancel;
                        Close();
                        return;
                    }
                }

                EditItemDialog editItemDialog
                    = new EditItemDialog(dataManager, item, folder);
                
                DialogResult = editItemDialog.ShowDialog();
            }
            Close();
        }

        private void HandleUserInput(object sender, EventArgs e)
        {
            if (nameTextBox.Text == string.Empty
                || (createFolder == false
                    && typeComboBox.Text == string.Empty))
            {
                okButton.Enabled = false;
            }
            else
            {
                okButton.Enabled = true;
            }
        }

        private void storeItem_Succeeded(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
            Close();
        }

        private void storeItem_Cancelled(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void storeItem_Failed(object sender, EventArgs e)
        {
            // TODO: From resource
            MessageBox.Show("Failed to store item");

            DialogResult = DialogResult.Abort;
            Close();
        }

        DataManager dataManager;
        ItemID folder;
        Dictionary<string, ItemType> types;
        bool createFolder;
    }
}