using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using PrivateDataCore;
using PrivateDataCore.ItemTypes;

namespace PrivateData.ItemControls
{
    public partial class CreditCardControl : UserControl, IEditItemControl
    {
        private CreditCardControl()
        {
            InitializeComponent();
        }

        public CreditCardControl(CreditCard item)
        {
            InitializeComponent();

            this.item = item;

            financialInstitutionTextBox.Text = item.FinancialInstitution;
            cardTypeComboBox.Text = item.CardType;
            cardNumberTextBox.Text = item.CardNumber;
            nameOnCardTextBox.Text = item.NameOnCard;
            expirationMonthComboBox.SelectedIndex = item.Expiration.Month - 1;
            expirationYearNumericUpDown.Value = item.Expiration.Year;
            cardVerificationCodeTextBox.Text = item.CardVerificationCode;
            pinTextBox.Text = item.PIN;
            customerSupportTextBox.Text = item.CustomerSupport;
            urlTextBox.Text = item.URL;
            usernameTextBox.Text = item.Username;
            passwordTextBox.Text = item.Password;
            notesTextBox.Text = item.Notes;
        }

        public bool Save()
        {
            if (textChanged)
            {
                item.FinancialInstitution = financialInstitutionTextBox.Text;
                item.CardType = cardTypeComboBox.Text;
                item.CardNumber = cardNumberTextBox.Text;
                item.NameOnCard = nameOnCardTextBox.Text;
                CreditCard.ExpirationDate date = new CreditCard.ExpirationDate();
                date.Month = expirationMonthComboBox.SelectedIndex + 1;
                date.Year = (int)expirationYearNumericUpDown.Value;
                item.Expiration = date;
                item.CardVerificationCode = cardVerificationCodeTextBox.Text;
                item.PIN = pinTextBox.Text;
                item.CustomerSupport = customerSupportTextBox.Text;
                item.URL = urlTextBox.Text;
                item.Username = usernameTextBox.Text;
                item.Password = passwordTextBox.Text;
                item.Notes = notesTextBox.Text;

                return true;
            }
            else
            {
                return false;
            }
        }

        private void HandleUserInput(object sender, EventArgs e)
        {
            textChanged = true;
        }

        private void tabControl_GotFocus(object sender, EventArgs e)
        {
            if (tabControl.TabPages[tabControl.SelectedIndex].Name == cardTabPage.Name)
            {
                financialInstitutionTextBox.Focus();
            }
            else if (tabControl.TabPages[tabControl.SelectedIndex].Name == securityTabPage.Name)
            {
                pinTextBox.Focus();
            }
            else if (tabControl.TabPages[tabControl.SelectedIndex].Name == webTabPage.Name)
            {
                urlTextBox.Focus();
            }
            else if (tabControl.TabPages[tabControl.SelectedIndex].Name == notesTabPage.Name)
            {
                notesTextBox.Focus();
            }
        }

        private CreditCard item;
        private bool textChanged;
    }
}
