using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;

using PrivateDataCore;
using PrivateDataCore.ItemTypes;

namespace PrivateData.ItemControls
{
    public partial class PictureControl : UserControl, IEditItemControl
    {
        public PictureControl()
        {
            InitializeComponent();
        }

        public PictureControl(Picture item)
        {
            this.item = item;

            InitializeComponent();

            pictureBox.Image = new Bitmap(item.PictureStream);
            pictureBox.SizeMode = PictureBoxSizeMode.StretchImage;
            ResizePictureBox();
        }

        public bool Save()
        {
            return item.IsImported;
        }

        private void ResizePictureBox()
        {
            if (pictureBox.Image != null)
            {
                SuspendLayout();

                double pictureAspectRatio
                    = (double)pictureBox.Image.Width
                        / (double)pictureBox.Image.Height;
                
                double tabPageAspectRatio
                    = (double)pictureTabPage.ClientSize.Width
                        / (double)pictureTabPage.ClientSize.Height;

                // Resize the picture box
                if (pictureAspectRatio > tabPageAspectRatio)
                {
                    // Picture's size is limited by the tab page's width
                    pictureBox.Width = pictureTabPage.ClientSize.Width;
                    pictureBox.Height
                        = (int)Math.Round(pictureBox.Width / pictureAspectRatio);
                }
                else
                {
                    // Picture's size is limited by the tab page's height
                    pictureBox.Height = pictureTabPage.ClientSize.Height;
                    pictureBox.Width
                        = (int)Math.Round(pictureBox.Height * pictureAspectRatio);
                }

                // And then center it
                pictureBox.Top
                    = pictureTabPage.ClientRectangle.Top
                        + (pictureTabPage.ClientSize.Height - pictureBox.Height) / 2;
                pictureBox.Left
                    = pictureTabPage.ClientRectangle.Left
                        + (pictureTabPage.ClientSize.Width - pictureBox.Width) / 2;

                ResumeLayout();
            }
        }

        private void tabControl_GotFocus(object sender, EventArgs e)
        {
            if (tabControl.TabPages[tabControl.SelectedIndex].Name == pictureTabPage.Name)
            {
                pictureBox.Focus();
            }
        }

        private void pictureTabPage_Resize(object sender, EventArgs e)
        {
            ResizePictureBox();
        }

        private Picture item;
    }
}

