using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

using PrivateDataCore;

namespace PrivateData
{
    public partial class LoginDialog : Form
    {
        public LoginDialog()
        {
            InitializeComponent();

            dataManager = new DataManager();
            UpdateFileNameLabel();
        }

        public string DataPath
        {
            get
            {
                return dataPath;
            }

            set
            {
                dataPath = value;
                UpdateFileNameLabel();
            }
        }

        public string Password
        {
            get
            {
                return passwordTextBox.Text;
            }

            set
            {
                passwordTextBox.Text = value;
            }
        }

        public DataManager DataManager
        {
            get
            {
                return dataManager;
            }
        }

        private void UpdateFileNameLabel()
        {
            fileNameLabel.Text = Path.GetFileName(dataPath);
        }

        private void Error(string message)
        {
            // TODO: From resource
            errorLabel.Text = message;
            passwordTextBox.Text = String.Empty;
            passwordTextBox.Focus();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            AsyncOperations.LoginOperation login
                = new AsyncOperations.LoginOperation(
                    dataManager,
                    dataPath,
                    passwordTextBox.Text);

            new AsyncOperationUI(
                login,
                this,
                new EventHandler(login_Succeeded),
                new EventHandler(login_Cancelled),
                new EventHandler(login_Failed));

            login.BeginExecute();
        }

        private void FileOpenMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                dataPath = openFileDialog.FileName;
                Settings.DataPath = dataPath;
                UpdateFileNameLabel();
            }
        }

        private void FileNewMenuItem_Click(object sender, EventArgs e)
        {
            CreateStorageDialog createStorageDialog
                = new CreateStorageDialog(dataManager);

            if (createStorageDialog.ShowDialog() == DialogResult.OK)
            {
                dataPath = createStorageDialog.DataPath;
                
                DialogResult = DialogResult.OK;
                Close();
            }
        }

        private void login_Succeeded(object sender, EventArgs e)
        {
            AsyncOperations.LoginOperation login
                = sender as AsyncOperations.LoginOperation;

            if (login.PasswordCorrect)
            {
                DialogResult = DialogResult.OK;
                Close();
            }
            else
            {
                Error("Incorrect password");
            }
        }

        private void login_Cancelled(object sender, EventArgs e)
        {
            Error("Login cancelled");
        }

        private void login_Failed(object sender, EventArgs e)
        {
            Error("Login failed");
        }

        private string dataPath;
        private DataManager dataManager;
    }
}
