using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace PrivateData
{
    public partial class NewPasswordDialog : Form
    {
        public NewPasswordDialog(bool changePassword)
        {
            InitializeComponent();

            strengthProgressBar.Maximum = strengthNeeded;
            oldPasswordTextBox.Focus();
            if (!changePassword)
            {
                SuspendLayout();
                oldPasswodLabel.Visible = false;
                oldPasswordTextBox.Visible = false;
                int shiftUp = oldPasswordTextBox.Bottom;
                newPasswordLabel.Top -= shiftUp;
                newPasswordTextBox.Top -= shiftUp;
                confirmPasswordLabel.Top -= shiftUp;
                confirmPasswordTextBox.Top -= shiftUp;
                strengthLabel.Top -= shiftUp;
                strengthProgressBar.Top -= shiftUp;
                okButton.Top -= shiftUp;
                cancelButton.Top -= shiftUp;
                newPasswordTextBox.Focus();
                ResumeLayout();
            }
        }

        public string NewPassword
        {
            get
            {
                return newPasswordTextBox.Text;
            }
        }

        public string OldPassword
        {
            get
            {
                return oldPasswordTextBox.Text;
            }
        }

        private decimal Entropy(char c)
        {
            if (c >= 'A' && c <= 'Z' || c >= 'a' && c <= 'z')
            {
                return 4.7004397181411M; // log2(26)
            }
            else if (c >= '0' && c <= '9')
            {
                return 3.3219280948874M; // log2(10)
            }
            else
            {
                // Assuming we have around 32 special characters and punctuation
                return 5.0M; // log2(32)
            }
        }

        private int PasswordStrength(string password)
        {
            decimal entropy = 0.0M;

            for (int i = 0; i < password.Length; i++)
            {
                entropy += Entropy(password[i]);
            }

            return (int)Math.Round(entropy);
        }

        private void passwordTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateOnInput();
        }

        private void confirmPasswordTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateOnInput();
        }

        private void UpdateOnInput()
        {
            int strength = PasswordStrength(newPasswordTextBox.Text);
            strengthProgressBar.Value = Math.Min(strength, strengthProgressBar.Maximum);

            okButton.Enabled
                = (strength >= strengthNeeded
                    && newPasswordTextBox.Text == confirmPasswordTextBox.Text);
        }

        const int strengthNeeded = 60;
    }
}