using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Diagnostics;

using PrivateDataCore;

namespace PrivateData
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [MTAThread]
        static void Main()
        {
            DataManager dataManager = OpenStorage();

            if (dataManager != null)
            {
                try
                {
                    Application.Run(new ItemListForm(dataManager));
                }
                finally
                {
                    dataManager.Close();
                }
            }
        }

        /// <summary>
        /// Opens the storage - either by letting the user log in or create
        /// a new storage.
        /// </summary>
        /// <returns>Data manager with an open storage, ready for work</returns>
        private static DataManager OpenStorage()
        {
            string dataPath = Settings.DataPath;
            DataManager dataManager = null;

            if (dataPath == null)
            {
                OpenOrCreateDialog openOrCreateDialog
                    = new OpenOrCreateDialog();

                DialogResult dialogResult = openOrCreateDialog.ShowDialog();
                if (dialogResult == DialogResult.Yes)
                {
                    // The user wishes to open an existing file
                    dataPath = openOrCreateDialog.DataPath;
                }
                else if (dialogResult == DialogResult.No)
                {
                    // The user wishes to create a new file
                    dataManager = new DataManager();
                    CreateStorageDialog createStorageDialog
                        = new CreateStorageDialog(dataManager);

                    if (createStorageDialog.ShowDialog() == DialogResult.OK)
                    {
                        dataPath = createStorageDialog.DataPath;
                    }
                    else
                    {
                        dataManager = null;
                    }
                }
            }

            // If there is a specific file to open (the user hasn't cancelled
            // out of all the dialogs), and we still haven't logged in as part
            // of file creation (so dataManager is null), let's do it now
            if (dataPath != null && dataManager == null)
            {
                LoginDialog loginDialog = new LoginDialog();
                loginDialog.DataPath = dataPath;
                if (loginDialog.ShowDialog() == DialogResult.OK)
                {
                    dataManager = loginDialog.DataManager;
                    dataPath = loginDialog.DataPath;
                }
            }

            // If we successfully logged in / created a new storage, let's save
            // the (potentially new) data path back to the registry
            if (dataManager != null)
            {
                Settings.DataPath = dataPath;
            }

            return dataManager;
        }
    }
}