// #define MULTIPLE_TESTS

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TestPrivateData
{
    public partial class TestPrivateDataForm : Form
    {
        public TestPrivateDataForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Performs a single Private Data test while providing feedback in the
        /// test output textbox
        /// </summary>
        /// <param name="test">Private Data test to execute</param>
        /// <returns>True if the test succeeded; false otherwise</returns>
        private bool PerformTest(Test test)
        {
            testStatusTextBox.Text = String.Empty;

            testOutputTextBox.SelectedText += String.Format("Testing {0}... ", test.Name);
            testOutputTextBox.Focus();
            testOutputTextBox.Select(testOutputTextBox.Text.Length, 0);
            testOutputTextBox.ScrollToCaret();

            bool result = test.ExecuteTest(new Test.SetTestStatusDelegate(SetTestStatus));

            testOutputTextBox.SelectedText += (result ? "SUCCEEDED\r\n" : "FAILED\r\n");
            testOutputTextBox.Focus();
            testOutputTextBox.Select(testOutputTextBox.Text.Length, 0);
            testOutputTextBox.ScrollToCaret();

            return result;
        }

        /// <summary>
        /// Sets test status information to the test status text box.
        /// Called during tests' execution to report controller.
        /// </summary>
        /// <param name="status">Status information to write out</param>
        private void SetTestStatus(string status)
        {
            testStatusTextBox.Text = status;
        }

        /// <summary>
        /// Executes all available Private Data tests
        /// </summary>
        private void RunTests()
        {
            int failures = 0;
            int totalTests = 0;

            foreach (Test test in Test.Tests)
            {
                totalTests++;

                if (!PerformTest(test))
                {
                    failures++;
                }
            }

            string results = string.Format("Tests executed: {0}, failures: {1}",
                totalTests,
                failures);
            MessageBox.Show(results, failures == 0 ? "Success" : "Failure");
        }

        private void executeTestsButton_Click(object sender, EventArgs e)
        {
#if MULTIPLE_TESTS
            for (int i = 0; i < 20; i++)
            {
#endif

                RunTests();

#if MULTIPLE_TESTS
            }
#endif
        }
    }
}