using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Security.Cryptography;

using PrivateDataCommon;
using PrivateDataCore;

namespace TestPrivateData.Tests
{
    class TestHMAC : Test
    {
        public override string Name
        {
            get
            {
                return "HMAC";
            }
        }

        public override bool ExecuteTest(Test.SetTestStatusDelegate setStatus)
        {
            byte[][] testKeysMD5 = new byte[][]
                {
                    new byte[] {
                        0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b,
                        0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b
                        },
                    Encoding.ASCII.GetBytes("Jefe"),
                    new byte[] {
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa
                        },
                    new byte[] {
                        0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 
                        0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 
                        0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18,
                        0x19
                        },
                    new byte[] {
                        0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c,
                        0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c
                        },
                    new byte[] {
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        },
                    new byte[] {
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        }
                };
            byte[][] testKeysSHA1 = new byte[][]
                {
                    new byte[] {
                        0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b,
                        0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b,
                        0x0b, 0x0b, 0x0b, 0x0b
                        },
                    Encoding.ASCII.GetBytes("Jefe"),
                    new byte[] {
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa
                        },
                    new byte[] {
                        0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 
                        0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x10, 
                        0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18,
                        0x19
                        },
                    new byte[] {
                        0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c,
                        0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c,
                        0x0c, 0x0c, 0x0c, 0x0c
                        },
                    new byte[] {
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        },
                    new byte[] {
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa,
                        }
                };
            byte[][] testData =
                {
                    Encoding.ASCII.GetBytes("Hi There"),
                    Encoding.ASCII.GetBytes("what do ya want for nothing?"),
                    new byte[] {
                        0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd,
                        0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd,
                        0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd,
                        0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd,
                        0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd,
                        0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd, 0xdd,
                        0xdd, 0xdd
                        },
                    new byte[] {
                        0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
                        0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
                        0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
                        0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
                        0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
                        0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
                        0xcd, 0xcd
                        },
                    Encoding.ASCII.GetBytes("Test With Truncation"),
                    Encoding.ASCII.GetBytes("Test Using Larger Than Block-Size Key - Hash Key First"),
                    Encoding.ASCII.GetBytes("Test Using Larger Than Block-Size Key and Larger Than One Block-Size Data"),
                };
            byte[][] expectedResultsMD5 =
                {
                    new byte[] {
                        0x92, 0x94, 0x72, 0x7a, 0x36, 0x38, 0xbb, 0x1c,
                        0x13, 0xf4, 0x8e, 0xf8, 0x15, 0x8b, 0xfc, 0x9d
                        },
                    new byte[] {
                        0x75, 0x0c, 0x78, 0x3e, 0x6a, 0xb0, 0xb5, 0x03,
                        0xea, 0xa8, 0x6e, 0x31, 0x0a, 0x5d, 0xb7, 0x38
                        },
                    new byte[] {
                        0x56, 0xbe, 0x34, 0x52, 0x1d, 0x14, 0x4c, 0x88,
                        0xdb, 0xb8, 0xc7, 0x33, 0xf0, 0xe8, 0xb3, 0xf6
                        },
                    new byte[] {
                        0x69, 0x7e, 0xaf, 0x0a, 0xca, 0x3a, 0x3a, 0xea,
                        0x3a, 0x75, 0x16, 0x47, 0x46, 0xff, 0xaa, 0x79
                        },
                    new byte[] {
                        0x56, 0x46, 0x1e, 0xf2, 0x34, 0x2e, 0xdc, 0x00, 
                        0xf9, 0xba, 0xb9, 0x95, 0x69, 0x0e, 0xfd, 0x4c
                        },
                    new byte[] {
                        0x6b, 0x1a, 0xb7, 0xfe, 0x4b, 0xd7, 0xbf, 0x8f, 
                        0x0b, 0x62, 0xe6, 0xce, 0x61, 0xb9, 0xd0, 0xcd
                        },
                    new byte[] {
                        0x6f, 0x63, 0x0f, 0xad, 0x67, 0xcd, 0xa0, 0xee,
                        0x1f, 0xb1, 0xf5, 0x62, 0xdb, 0x3a, 0xa5, 0x3e
                        }
                };
            byte[][] expectedResultsSHA1 =
                {
                    new byte[] {
                        0xb6, 0x17, 0x31, 0x86, 0x55, 0x05, 0x72, 0x64, 
                        0xe2, 0x8b, 0xc0, 0xb6, 0xfb, 0x37, 0x8c, 0x8e, 
                        0xf1, 0x46, 0xbe, 0x00
                        },
                    new byte[] {
                        0xef, 0xfc, 0xdf, 0x6a, 0xe5, 0xeb, 0x2f, 0xa2, 
                        0xd2, 0x74, 0x16, 0xd5, 0xf1, 0x84, 0xdf, 0x9c, 
                        0x25, 0x9a, 0x7c, 0x79
                        },
                    new byte[] {
                        0x12, 0x5d, 0x73, 0x42, 0xb9, 0xac, 0x11, 0xcd,
                        0x91, 0xa3, 0x9a, 0xf4, 0x8a, 0xa1, 0x7b, 0x4f, 
                        0x63, 0xf1, 0x75, 0xd3
                        },
                    new byte[] {
                        0x4c, 0x90, 0x07, 0xf4, 0x02, 0x62, 0x50, 0xc6, 
                        0xbc, 0x84, 0x14, 0xf9, 0xbf, 0x50, 0xc8, 0x6c, 
                        0x2d, 0x72, 0x35, 0xda
                        },
                    new byte[] {
                        0x4c, 0x1a, 0x03, 0x42, 0x4b, 0x55, 0xe0, 0x7f, 
                        0xe7, 0xf2, 0x7b, 0xe1, 0xd5, 0x8b, 0xb9, 0x32, 
                        0x4a, 0x9a, 0x5a, 0x04
                        },
                    new byte[] {
                        0xaa, 0x4a, 0xe5, 0xe1, 0x52, 0x72, 0xd0, 0x0e, 
                        0x95, 0x70, 0x56, 0x37, 0xce, 0x8a, 0x3b, 0x55, 
                        0xed, 0x40, 0x21, 0x12
                        },
                    new byte[] {
                        0xe8, 0xe9, 0x9d, 0x0f, 0x45, 0x23, 0x7d, 0x78,
                        0x6d, 0x6b, 0xba, 0xa7, 0x96, 0x5c, 0x78, 0x08,
                        0xbb, 0xff, 0x1a, 0x91
                        }
                };

            if (testKeysMD5.Length != testKeysSHA1.Length
                    || testKeysMD5.Length != testData.Length
                    || testKeysMD5.Length != expectedResultsMD5.Length
                    || testKeysMD5.Length != expectedResultsSHA1.Length)
            {
                throw new Exception("Invalid test data in TestHMAC!");
            }

            HashAlgorithm md5 = MD5.Create();
            HashAlgorithm sha1 = SHA1.Create();
            MethodInfo HMAC = Type.GetType("PrivateDataCore.Crypto, Private Data Core")
                .GetMethod("HMAC",
                           BindingFlags.InvokeMethod
                                | BindingFlags.Static
                                | BindingFlags.Public
                                | BindingFlags.NonPublic);

            for (int test = 0; test < testData.Length; test++)
            {
                byte[] resultMD5 = (byte[])HMAC.Invoke(null, new object[] { md5, testKeysMD5[test], testData[test] });
                if (!Utils.ByteArraysMatch(resultMD5, expectedResultsMD5[test]))
                {
                    return false;
                }

                byte[] resultSHA1 = (byte[])HMAC.Invoke(null, new object[] { sha1, testKeysSHA1[test], testData[test] });
                if (!Utils.ByteArraysMatch(resultSHA1, expectedResultsSHA1[test]))
                {
                    return false;
                }
            }

            return true;
        }
    }
}
