using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace TestPrivateData
{
    class Timer
    {
        [DllImport("coredll.dll")]
        extern static int QueryPerformanceCounter(ref long perfCounter);

        [DllImport("coredll.dll")]
        extern static int QueryPerformanceFrequency(ref long frequency);

        /// <summary>
        /// Static constructor to obtain the frequency
        /// </summary>
        static Timer()
        {
            if (QueryPerformanceFrequency(ref frequency) == 0)
            {
                throw new ApplicationException();
            }

            // Convert from ticks/second to ticks/millisecond
            frequency /= 1000;
        }

        /// <summary>
        /// Starts the timer
        /// </summary>
        public void Start()
        {
            if (QueryPerformanceCounter(ref start) == 0)
            {
                throw new ApplicationException();
            }
        }

        /// <summary>
        /// Stops the timer
        /// </summary>
        /// <returns>Time elapsed, in milliseconds</returns>
        public Int64 Stop()
        {
            Int64 stop = 0;
            if (QueryPerformanceCounter(ref stop) == 0)
            {
                throw new ApplicationException();
            }

            return (stop - start) / frequency;
        }

        static private Int64 frequency;
        private Int64 start;
    }
}